﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HR;
using Interfaces;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace SimplestTestDemo
{
    [TestClass]
    public class HrTester
    {
        /// <summary>
        /// 这个测试没有任何Assert，能跑通过就认为OK
        /// </summary>
        [TestMethod]
        public void TestGetAllEmployees()
        {
            IHr hr = new HrManager();
            IEnumerable<Employee_Get> emps = hr.GetAllEmployees();
            foreach (var emp in emps)
            {
                //此输出有利于你在测试对话框中检查相关内容
                Console.WriteLine("工号: " + emp.EmpNo + "\n\t姓名: " + emp.Name + "\n\t生日: " + emp.Birthday + "\n\t描述: " +
                                  emp.Descriptions + "\n\t状态: " + emp.Status + "\n\t工资: " + emp.Salary);
            }
        }

        /// <summary>
        /// 获取一个不存在的用户，期待程序抛出一个异常
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(DemoException))]
        public void TestGetEmployee()
        {
            IHr hr = new HrManager();
            hr.GetEmployee("does not exist");
        }

        /// <summary>
        /// 多个测试细节合并为一个：
        ///     尝试增加一个已存在的工号 - 期待抛出异常
        ///     增加一个不存在的工号 - 应该成功
        ///     检查新增员工是否存在 - 应该存在
        ///     用这个工号获取员工信息 - 应该成功，并检查返回值
        ///     给此员工加薪 - 期待抛出异常
        ///     转正此员工 - 应该成功
        ///     给此员工加薪 - 应该成功
        ///     再次获取员工信息 - 应该成功，并比较薪资
        ///     员工离职 - 应该成功
        ///     删除员工 - 应该成功
        ///     检查员工共是否存在 - 应该不存在
        /// </summary>
        [TestMethod]
        public void IntegrationTest()
        {
            IHr hr = new HrManager();

            const string empNoToTest = "0100";

            try
            {
                hr.AddEmployee(new Employee_Add() { EmpNo = "0001", Name = "Somebody", Birthday = DateTime.Now });
                Assert.Fail("尝试增加已存在工号应当抛出异常");
            }
            catch (Exception ex)
            {
                Assert.IsInstanceOfType(ex, typeof(DemoException));
            }

            Employee_Add empToAdd = new Employee_Add
                                        {
                                            EmpNo = empNoToTest,
                                            Name = "大虞",
                                            Birthday = new DateTime(1979, 2, 11),
                                            Descriptions = "高富帅"
                                        };


            hr.AddEmployee(empToAdd);

            Assert.AreEqual(hr.IsEmployeeExist(empNoToTest), true);

            Employee_Get empJustAdded = hr.GetEmployee(empNoToTest);
            Assert.IsTrue(empJustAdded.EmpNo==empToAdd.EmpNo
                && empJustAdded.Name==empToAdd.Name
                && empJustAdded.Birthday==empToAdd.Birthday
                && empJustAdded.Descriptions==empToAdd.Descriptions);

            try
            {
                hr.IncreaseSalary(empNoToTest, 800.00m);
                Assert.Fail("尝试给非正式员工加薪应当抛出异常");
            }
            catch (Exception ex)
            {
                Assert.IsInstanceOfType(ex, typeof(DemoException));
            }

            hr.BecomeRegular(empNoToTest);

            hr.IncreaseSalary(empNoToTest, 800.00m);

            Employee_Get empIncreasedSalary = hr.GetEmployee(empNoToTest);
            Assert.AreEqual(empIncreasedSalary.Salary, empJustAdded.Salary + 800.00m);

            hr.BecomeLeave(empNoToTest);

            hr.DeleteEmployee(empNoToTest);

            Assert.AreEqual(hr.IsEmployeeExist(empNoToTest), false);
        }
    }
}
